/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   www.cs.sandia.gov/~sjplimp/lammps.html
   Steve Plimpton, sjplimp@sandia.gov, Sandia National Laboratories

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

/* ----------------------------------------------------------------------
   Contributing author: Ravi Agrawal (Northwestern U)
------------------------------------------------------------------------- */

#include "math.h"
#include "string.h"
#include "fix_cavity.h"
#include "atom.h"
#include "domain.h"
#include "update.h"
#include "respa.h"
#include "error.h"

#define NONE     0
#define SPHERE   1
#define CYLINDER 2

/* ---------------------------------------------------------------------- */

FixCavity::FixCavity(int narg, char **arg) : Fix(narg, arg)
{
  if (narg < 4) error->all("Illegal fix indent command");
  k = atof(arg[3]);

  // set input line defaults
  istyle = NONE;

  // read options from end of input line

  options(narg-4,&arg[4]);

  //ntimestep_initial = update->ntimestep;
}

/* ---------------------------------------------------------------------- */

int FixCavity::setmask()
{
  int mask = 0;
  mask |= POST_FORCE;
  mask |= POST_FORCE_RESPA;
  return mask;
}

/* ---------------------------------------------------------------------- */

void FixCavity::init()
{
  if (strcmp(update->integrate_style,"respa") == 0)
    nlevels_respa = ((Respa *) update->integrate)->nlevels;
}

/* ---------------------------------------------------------------------- */

void FixCavity::setup()
{
  if (strcmp(update->integrate_style,"verlet") == 0)
    post_force(1);
  else {
    ((Respa *) update->integrate)->copy_flevel_f(nlevels_respa-1);
    post_force_respa(1,nlevels_respa-1,0);
    ((Respa *) update->integrate)->copy_f_flevel(nlevels_respa-1);
  }
	delta = 0.0;
	scale_delta = 1.0 / update->nsteps;
}

/* ---------------------------------------------------------------------- */

void FixCavity::post_force(int vflag)
{
	double **x = atom->x;
	double **f = atom->f;
	int *mask = atom->mask;
	int nlocal = atom->nlocal;

	double delx,dely,delz,r,dr,fmag;

	// grow the radius
	/*double delta = (update->ntimestep - ntimestep_initial);
	if (update->nsteps) delta /= update->nsteps*0.9;
	else delta = 0.0;
	if (update->ntimestep > update->nsteps*0.9) delta = 1.0;*/
	delta += scale_delta;
	double r_curr = delta * r0;
	
  // spherical cavity
  if (istyle == SPHERE) {
    for (int i = 0; i < nlocal; i++)
      if (mask[i] & groupbit) {
				delx = x[i][0] - x0;
				dely = x[i][1] - y0;
				delz = x[i][2] - z0;
				r = sqrt(delx*delx + dely*dely + delz*delz);
				dr = r - r_curr;
				if (dr < 0.0) {
					fmag = k*dr*dr/r;
					f[i][0] += delx*fmag;
					f[i][1] += dely*fmag;
					f[i][2] += delz*fmag;
				}
      }
  } else {
    // cylindrical cavity
    for (int i = 0; i < nlocal; i++)
      if (mask[i] & groupbit) {
				if (cdim == 0) {
					delx = 0;
					dely = x[i][1] - c1;
					delz = x[i][2] - c2;
				} else if (cdim == 1) {
					delx = x[i][0] - c1;
					dely = 0;
					delz = x[i][2] - c2;
				} else {
					delx = x[i][0] - c1;
					dely = x[i][1] - c2;
					delz = 0;
				}
				r = sqrt(delx*delx + dely*dely + delz*delz);
				dr = r - r_curr;
				if (dr < 0.0) {
					fmag = k*dr*dr/r;
					f[i][0] += delx*fmag;
					f[i][1] += dely*fmag;
					f[i][2] += delz*fmag;
				}
			}
  }
}

/* ---------------------------------------------------------------------- */

void FixCavity::post_force_respa(int vflag, int ilevel, int iloop)
{
  if (ilevel == nlevels_respa-1) post_force(vflag);
}

/* ----------------------------------------------------------------------
   parse optional parameters at end of fix cavity input line 
------------------------------------------------------------------------- */

void FixCavity::options(int narg, char **arg)
{
  if (narg < 0) error->all("Illegal fix cavity command");

  int iarg = 0;
  while (iarg < narg) {
    if (strcmp(arg[iarg],"sphere") == 0) {
      if (iarg+5 > narg) error->all("Illegal fix cavity command");
      x0 = atof(arg[iarg+1]);
      y0 = atof(arg[iarg+2]);
      z0 = atof(arg[iarg+3]);
      r0 = atof(arg[iarg+4]);
      istyle = SPHERE;
      iarg += 5;
    } else if (strcmp(arg[iarg],"cylinder") == 0) {
      if (iarg+5 > narg) error->all("Illegal fix cavity command");
     if (strcmp(arg[iarg+1],"x") == 0) cdim = 0;
      else if (strcmp(arg[iarg+1],"y") == 0) cdim = 1;
      else if (strcmp(arg[iarg+1],"z") == 0) cdim = 2;
      else error->all("Illegal fix cavity command");
      c1 = atof(arg[iarg+2]);
      c2 = atof(arg[iarg+3]);
      r0 = atof(arg[iarg+4]);
      istyle = CYLINDER;
      iarg += 5;
    } else error->all("Incorrect args in fix cavity options");
  }
}
